const util = require('util');
const fs = require('fs');
const childProcess = require('child_process');

const exec = util.promisify(childProcess.exec);

class OnePassword {
	constructor() {}

	async signin() {
		return new Promise((resolve, reject) => {
			let data = '';
			var p = childProcess.spawn('op', ['signin'], { stdio: [fs.openSync('/dev/tty'), 'pipe', 'inherit'] });
			p.stdout.setEncoding('utf-8');
			p.stdout.on('data', function(chunk) {
				data += chunk;
			});

			p.on('close', async function(code) {
				var m = data.match('export (?<key>OP_SESSION_.*)="(?<val>[^"]+)"');
				if (m) {
					process.env[m.groups.key] = m.groups.val;
				}
				resolve(data);
			});
			p.on('error', function(err) {
				reject(err);
			});
		});

	}

	async get(key) {
		return new Promise((resolve, reject) => {
			let data = '';
			var p = childProcess.spawn('op', ['get', 'item', key], { stdio: ['inherit', 'pipe', 'inherit'] });
			p.stdout.setEncoding('utf-8');
			p.stdout.on('data', function(chunk) {
				data += chunk;
			});

			p.on('close', async function(code) {
				const item = JSON.parse(data);

				let username, password;
				item.details.fields.forEach(field => {
					if (field.designation === 'username') {
						username = field.value;
					}
					if (field.designation === 'password') {
						password = field.value;
					}
				});

				resolve({
					username: username,
					password: password
				});
			});
			p.on('error', function(err) {
				reject(err);
			});
		});
	}
}

module.exports = OnePassword;
